@@ -366,7 +366,7 @@ void RPZIXFRTracker(const std::vector<ComboAddress>& masters, boost::optional<DN
   // If oldZone failed to load its getRefresh() returns 0, protect against that
   uint32_t refresh = std::max(refreshFromConf ? refreshFromConf : oldZone->getRefresh(), 10U);
   DNSName zoneName = oldZone->getDomain();
-  std::string polName = oldZone->getName().empty() ? oldZone->getName() : zoneName.toString();
+  std::string polName = !oldZone->getName().empty() ? oldZone->getName() : zoneName.toStringNoDot();
 
   while (!sr) {
     /* if we received an empty sr, the zone was not really preloaded */
@@ -456,7 +456,15 @@ void RPZIXFRTracker(const std::vector<ComboAddress>& masters, boost::optional<DN
     try {
       g_log<<Logger::Info<<"Processing "<<deltas.size()<<" delta"<<addS(deltas)<<" for RPZ "<<zoneName<<endl;
 
+      if (luaconfsLocal->generation != configGeneration) {
+        g_log<<Logger::Info<<"A more recent configuration has been found, stopping the existing RPZ update thread for "<<zoneName<<endl;
+        return;
+      }
       oldZone = luaconfsLocal->dfe.getZone(zoneIdx);
+      if (!oldZone || oldZone->getDomain() != zoneName) {
+        g_log<<Logger::Info<<"This policy is no more, stopping the existing RPZ update thread for "<<zoneName << endl;
+        return;
+      }
       /* we need to make a _full copy_ of the zone we are going to work on */
       std::shared_ptr<DNSFilterEngine::Zone> newZone = std::make_shared<DNSFilterEngine::Zone>(*oldZone);
       /* initialize the current serial to the last one */
@@ -527,6 +535,10 @@ void RPZIXFRTracker(const std::vector<ComboAddress>& masters, boost::optional<DN
          but we don't want to touch anything else, especially other zones,
          since they might have been updated by another RPZ IXFR tracker thread.
       */
+      if (luaconfsLocal->generation != configGeneration) {
+        g_log<<Logger::Info<<"A more recent configuration has been found, stopping the existing RPZ update thread for "<<zoneName<<endl;
+        return;
+      }
       g_luaconfs.modify([zoneIdx, &newZone](LuaConfigItems& lci) {
                           lci.dfe.setZone(zoneIdx, newZone);
                         });
